/**
 * Node Web Serial Ponyfill v0.5.3-beta.3
 * Copyright 2022-2024 jerrylum <me@jerryio.com>
 */

import { SerialPort } from 'serialport';
import { ReadableStream, WritableStream } from 'web-streams-polyfill/ponyfill';
import readline from 'readline';
import util from 'util';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

class NodeUnderlyingSource {
    constructor(port_, adapter_) {
        this.port_ = port_;
        this.adapter_ = adapter_;
    }
    handleDisconnection(controller) {
        if (this.adapter_.readable_)
            controller.error(new Error("The device has been lost."));
        else if (!controller._closeRequested && controller._controlledReadableStream._state === 'readable')
            // HACK: avoid "The stream is not in a state that permits close" error
            controller.close();
    }
    start(_controller) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.port_.read(0);
        });
    }
    pull(controller) {
        if (!this.port_.isOpen) {
            this.handleDisconnection(controller);
            return;
        }
        const onClose = () => this.handleDisconnection(controller);
        this.adapter_.controllerQueue_.push(controller);
        this.port_.once("close", onClose);
        this.port_.once("data", (_stream) => __awaiter(this, void 0, void 0, function* () {
            this.port_.removeListener("close", onClose);
        }));
    }
    cancel() {
        if (!this.port_.isOpen && this.adapter_.readable_)
            throw new Error("The device has been lost.");
    }
}
class NodeUnderlyingSink {
    constructor(port_) {
        this.port_ = port_;
    }
    write(chunk) {
        return new Promise((resolve, reject) => {
            if (!this.port_.isOpen) {
                resolve(); // ignored
                return;
            }
            this.port_.write(chunk, (err) => {
                if (err)
                    reject(err);
            });
            this.port_.drain((err) => {
                if (err)
                    reject(err);
                else
                    resolve();
            });
        });
    }
}
class NodeSerialPortAdapter extends EventTarget {
    constructor(info) {
        super();
        this.readBuffer_ = Buffer.from([]);
        this.controllerQueue_ = [];
        this.info_ = info;
    }
    get readable() {
        var _a;
        if (!this.readable_ && ((_a = this.port_) === null || _a === void 0 ? void 0 : _a.isOpen)) {
            this.readable_ = new ReadableStream(new NodeUnderlyingSource(this.port_, this));
        }
        return this.readable_;
    }
    get writable() {
        var _a;
        if (!this.writable_ && ((_a = this.port_) === null || _a === void 0 ? void 0 : _a.isOpen)) {
            this.writable_ = new WritableStream(new NodeUnderlyingSink(this.port_));
        }
        return this.writable_;
    }
    open(options) {
        return new Promise((resolve, reject) => {
            var _a;
            if (this.port_)
                throw new Error("Failed to execute 'open' on 'SerialPort': The port is already open.");
            this.port_ = new (options.upstream || SerialPort)({
                path: this.info_.path,
                baudRate: options.baudRate,
                dataBits: options.dataBits,
                stopBits: options.stopBits,
                autoOpen: true,
                parity: options.parity,
                highWaterMark: (_a = options.bufferSize) !== null && _a !== void 0 ? _a : 65536
            }, (err) => {
                if (err) {
                    // XXX: Using error message in Node.js instead of following the error message in dom
                    // DOMException: Failed to open serial port.
                    reject(err);
                }
                else if (this.port_) {
                    this.port_.on("close", this.closePortEvent.bind(this));
                    this.port_.on("data", this.receiveDataEvent.bind(this));
                    this.dispatchEvent(new Event("open"));
                    // if (this.onconnect) this.onconnect(new Event("connect"));
                    resolve();
                }
            });
        });
    }
    close() {
        return new Promise((resolve) => {
            if (!this.port_)
                throw new Error("Failed to execute 'close' on 'SerialPort': The port is already closed.");
            // XXX: Not implemented the following error
            // TypeError: Failed to execute 'close' on 'SerialPort': Cannot cancel a locked stream
            this.readable_ = undefined;
            this.writable_ = undefined;
            this.port_.close(() => {
                this.port_ = undefined;
                this.controllerQueue_ = [];
                resolve();
            });
        });
    }
    getInfo() {
        return {
            serialNumber: this.info_.serialNumber,
            manufacturer: this.info_.manufacturer,
            locationId: this.info_.locationId,
            vendorId: this.info_.vendorId,
            vendor: undefined,
            productId: this.info_.productId,
            product: undefined,
            usbVendorId: Number("0x" + (this.info_.vendorId || "0")),
            usbProductId: Number("0x" + (this.info_.productId || "0"))
        };
    }
    setSignals(signals) {
        return new Promise((resolve, reject) => {
            if (!this.port_)
                throw new Error("Failed to execute 'setSignals' on 'SerialPort': The port is not open.");
            this.port_.set({ dtr: signals.dataTerminalReady, rts: signals.requestToSend, brk: signals.break }, (err) => {
                if (err)
                    reject(err);
                else
                    resolve();
            });
        });
    }
    closePortEvent() {
        this.dispatchEvent(new Event("close"));
        // if (this.ondisconnect) this.ondisconnect(new Event("disconnect"));
    }
    receiveDataEvent(stream) {
        let controller = this.controllerQueue_.shift();
        if (controller) {
            let data = Buffer.concat([this.readBuffer_, stream]);
            let ab = data.buffer.slice(data.byteOffset, data.byteOffset + data.byteLength);
            controller.enqueue(new Uint8Array(ab));
            this.readBuffer_ = Buffer.from([]);
        }
        else {
            this.readBuffer_ = Buffer.concat([this.readBuffer_, stream]);
        }
    }
}

const rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout
});
const prompt = util.promisify(rl.question).bind(rl);

class NodeSerialAdapter extends EventTarget {
    constructor() {
        super(...arguments);
        this.selectedPorts = [];
    }
    listPorts(options) {
        return __awaiter(this, void 0, void 0, function* () {
            let ports = [];
            let portsInfo = yield SerialPort.list();
            for (let info of portsInfo) {
                ports.push(new NodeSerialPortAdapter(info));
            }
            return ports.filter(port => {
                return (options === null || options === void 0 ? void 0 : options.filters)
                    ? options.filters.some(filter => filter.usbVendorId === port.getInfo().usbVendorId &&
                        (!filter.usbProductId || filter.usbProductId === port.getInfo().usbProductId))
                    : true;
            });
        });
    }
    findPort(portPath) {
        return __awaiter(this, void 0, void 0, function* () {
            let ports = yield this.listPorts();
            return ports.find(port => port.info_.path === portPath);
        });
    }
    /**
     * In a browser: it returns connected ports that the site already has access to.
     * In Node.js: it returns all ports that the user has selected in requestPort();
     *
     * @returns the list of ports that the user has selected in requestPort();
     */
    getPorts() {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.listPorts()).filter(port => { var _a; return this.selectedPorts.includes((_a = port.info_.pnpId) !== null && _a !== void 0 ? _a : ""); });
        });
    }
    requestPort(options) {
        return __awaiter(this, void 0, void 0, function* () {
            let ports = yield this.listPorts(options);
            console.log("\nPlease select a port.\n------------------------------");
            for (let i = 0; i < ports.length; i++) {
                let port = ports[i];
                let info = port.info_;
                let friendlyName = info.friendlyName;
                if (friendlyName !== undefined)
                    console.log(`${i}: ${friendlyName}`);
                else
                    console.log(`${i}: ${info.serialNumber} - ${info.manufacturer} (${info.path})`);
            }
            if (ports.length === 0) {
                console.log("(no available ports)");
            }
            console.log("------------------------------");
            if (ports.length === 0) {
                var ans = yield prompt('Enter "r" to reload the list: ');
            }
            else if (ports.length === 1) {
                var ans = yield prompt(`Enter 0 to the port or "r" to reload the list: `);
            }
            else {
                var ans = yield prompt(`Enter 0 ~ ${ports.length - 1} to select a port or "r" to reload the list: `);
            }
            if (ans === 'r') {
                return this.requestPort(options);
            }
            else if (ans !== "" && ports[Number(ans)]) {
                let port = ports[Number(ans)];
                if (port.info_.pnpId !== undefined)
                    this.selectedPorts.push(port.info_.pnpId);
                return port;
            }
            throw new Error("No port selected by the user.");
        });
    }
}

const serial = new NodeSerialAdapter();

export { serial };
//# sourceMappingURL=node-web-serial-ponyfill.es.js.map
